from iteration import Transfer, fixpoint_iteration
from syntax import *
from cfg import Cfg
from example_opsem import factorial

@dataclass
class Number_step:
    bound: int | None
    def __str__(self):
        if self.bound is None: return "+∞"
        else: return str(self.bound)

type S = Number_step | None

class Analysis(Transfer[S]):
    def bottom(self) -> S:
        return None

    def init_state(self) -> S:
        return Number_step(0)

    def join(self,s1:S,s2:S) -> S:
        if s1 is None: return s2
        if s2 is None: return s1
        if s1.bound is None: return s1
        if s2.bound is None: return s2
        return Number_step(max(s1.bound, s2.bound))

    def widen(self, s1:S, s2:S) -> S:
        if s1 is None: return s2
        if s2 is None: return s1
        if s1.bound is None: return s1
        if s2.bound is None: return s2
        if s1.bound < s2.bound: return Number_step(None)
        return s1

    def included(self,s1: S,s2: S) -> bool:
        if s1 is None: return True
        if s2 is None: return False
        if s2.bound is None: return True
        if s1.bound is None: return False
        return s1.bound <= s2.bound

    def tr_skip(self,s: S) -> S:
        if s is None: return None
        if s.bound is None: return s
        return Number_step(s.bound + 1)

    def tr_set(self,s: S,v: str,e: ArithExpr) -> S:
        return self.tr_skip(s)

    def tr_test(self,s: S,c: BoolExpr) -> S:
        return self.tr_skip(s)

    def tr_err(self,s: S,e: Expr) -> S:
        return self.tr_skip(s)

res = fixpoint_iteration(Analysis(),Cfg(factorial))

for (node, nb) in res.items():
    print(f'{node} -> {nb}')
