"""
Operational semantics for the Why language

Provides evaluation functions for the three syntactic classes,
based on a environment mapping variables to integer values.
These functions return None when an error occur somewhere (Big step semantics)
"""

from syntax import *

def eval_aexp(env: dict[str,int], exp: ArithExpr) -> int | None:
    """evaluates an arithmetic expression"""
    match exp:
        case AECst(value):
            raise NotImplementedError
        case AEVar(var):
            raise NotImplementedError
        case AEUop(uop,expr):
            raise NotImplementedError
        case AEBop(bop,left_expr,right_expr):
            raise NotImplementedError
        case _: assert False

def eval_bexp(env: dict[str,int], exp:BoolExpr) -> bool | None:
    """evaluates a boolean expression"""
    match exp:
        case BEPlain(aexpr):
            raise NotImplementedError
        case BEEq(left_expr,right_expr):
            raise NotImplementedError
        case BELeq(left_expr,right_expr):
            raise NotImplementedError
        case BENeg(expr):
            raise NotImplementedError
        case _: assert False

def eval_instr(env: dict[str,int], instr: Instr) -> dict[str,int] | None:
    """evaluates an instruction"""
    match instr:
        case ISkip():
            raise NotImplementedError
        case ISet(var,expr):
            raise NotImplementedError
        case ISeq(first,second):
            raise NotImplementedError
        case ICond(cond,true_branch,false_branch):
            raise NotImplementedError
        case ILoop(cond,body):
            raise NotImplementedError
        case _: assert False
